@@ -593,7 +593,13 @@ pl_tuple_heap(VALUE c, VALUE tuple)
 	rb_raise(pl_ePLruby, "Invalid descriptor");
     }
     if (TYPE(c) != T_ARRAY) {
-	c = rb_Array(c);
+	if (NIL_P(c) || (TYPE(c) == T_STRING && !RSTRING_LEN(c))) {
+	    c = rb_ary_new2(1);
+	    rb_ary_push(c, rb_str_new2(""));
+	}
+	else {
+	    c = rb_Array(c);
+	}
     }
     if (TYPE(c) != T_ARRAY || !RARRAY_PTR(c)) {
         rb_raise(pl_ePLruby, "expected an Array");
@@ -1015,11 +1021,11 @@ pl_convert_arg(Datum value, Oid typoid, FmgrInfo *finfo, Oid typelem,
         if (NIL_P(klass)) {
             klass = rb_hash_aref(plruby_conversions, vid);
             if (NIL_P(klass)) {
-                st_insert(RHASH(plruby_classes)->tbl, vid, Qfalse);
+                st_insert(RHASH_TBL(plruby_classes), vid, Qfalse);
             }
             else {
                 klass = rb_const_get(rb_cObject, NUM2INT(klass));
-                st_insert(RHASH(plruby_classes)->tbl, vid, klass);
+                st_insert(RHASH_TBL(plruby_classes), vid, klass);
             }
         }
         if (RTEST(klass)) {
@@ -1064,8 +1070,13 @@ create_array(index, ndim, dim, p, prodesc, curr, typoid)
                                   prodesc->arg_len[curr]);
             tmp = pl_convert_arg(itemvalue, typoid,
                                  &prodesc->arg_func[curr], (Datum)0, -1);
+#ifdef att_addlength_pointer
+	    *p = att_addlength_pointer(*p, prodesc->arg_len[curr], PointerGD(*p));
+	    *p = (char *) att_align_nominal(*p, prodesc->arg_align[curr]);
+#else
             *p = att_addlength(*p, prodesc->arg_len[curr], PointerGD(*p));
             *p = (char *) att_align(*p, prodesc->arg_align[curr]);
+#endif
             rb_ary_push(res, tmp); 
         }
         else {
@@ -1401,7 +1412,7 @@ plruby_return_value(struct pl_thread_st *plth, pl_proc_desc *prodesc,
 					    pl_tuple_put, tuple);
 		    }
 		    else {
-			res = rb_block_call(pl_mPLtemp, args->id, 1, args->ary,
+			res = rb_block_call(pl_mPLtemp, args->id, 1, RARRAY_PTR(args->ary),
 					    pl_tuple_put, tuple);
 		    }
 		}
